require("class")


-- Use OpenMenu (commands.lua) for console command
function ToggleMenu()
	if not GetPlayer().components.testtoolcontroller then
		print("Missing TTC component.")
		return
	end

	local active_screen = TheFrontEnd:GetActiveScreen()
	
	if active_screen and active_screen.displayed then
		active_screen:Close()
		return
	end
	
	if not IsHUDPaused() then
		SetHUDPause(true)
		TheFrontEnd:PushScreen(TestScreen(true))
	end
end

local function IsValidTile(point)
	local ground = TheSim:FindFirstEntityWithTag("ground")
	
	if ground and point then
		local tile = ground.Map:GetTileAtPoint(point.x, point.y, point.z)
		if tile ~= GROUND.IMPASSIBLE then
			return true
		end
	end	

	return false
end

--[[
	New function IsKeyBinding to test if currently pressed keys match desired key binding
	Created by: WrathOf
--]]
local function IsKeyBinding(key, modifiers)
	if TheInput:IsKeyDown(key) then
		if modifiers then
			for k,v in ipairs(modifiers) do
				if not TheInput:IsKeyDown(v) then return false end
			end
			return true
		else
			return not ( TheInput:IsKeyDown(KEY_CTRL) or TheInput:IsKeyDown(KEY_SHIFT) or TheInput:IsKeyDown(KEY_ALT) )
		end
	else
	  return false
	end
end

-- \ key code missing from constants.lua
KEY_BACKSLASH = 92

local TestToolController = Class(function(self, inst)

	self.inst = inst

	self.entityPrefab = nil
	self.itemPrefab = nil
	self.foodPrefab = nil
	self.objectPrefab = nil
	self.item_2_Prefab = nil

	self.speed_multiplier = nil

	self.bookmarked_point = nil

	self.inputhandlers = {}

-----------------------
-- Teleporting
table.insert(self.inputhandlers, TheInput:AddGeneralControlHandler(function(control, value) self:OnControl(control, value) end))
-----------------------	chromiumboy
	-- God mode
	table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_G, function()
		if not IsPaused() then
		if IsKeyBinding(KEY_G, {KEY_CTRL}) then
			ToggleGodMode()
		end
		end
	end))
	
	table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_Z, function()
		if not IsPaused() then
		-- Change season
		if IsKeyBinding(KEY_Z, {KEY_CTRL}) then
			NextSeason()
		end
		end
	end))

-- Key Bindings
 ---
 ---
----
-----
-----
	-- Delete (mouse)
	table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_P, function()
		if not IsPaused() then
		if IsKeyBinding(KEY_P, {KEY_SHIFT}) then	
			local target = TheInput:GetWorldEntityUnderMouse()
			
			if target and not target:HasTag("player") then
				if target.components.crop then
					target.components.crop:Mature()
					target.components.crop:Harvest(self.inst)
				end

				target:Remove()
			end	
		end
		end
  end))
-----
	table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_F, function()
		if not IsPaused() then
		-- Freebuild
		if IsKeyBinding(KEY_F, {KEY_CTRL}) then
			Freebuild()
		end
		end
	end))
	-----
	-- Time scale
  	table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_V, function()
		if not IsPaused() then
		if IsKeyBinding(KEY_V, {KEY_CTRL}) then
			ToggleTimeScale()
		end
		end
	end))
	
	-- Next day
  	table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_F, function()
		if not IsPaused() then
		if IsKeyBinding(KEY_F, {KEY_SHIFT}) then
			NextDay()
		end
		end
	end))
	
	-- Next phase
	table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_G, function()
		if not IsPaused() then
		if IsKeyBinding(KEY_G, {KEY_SHIFT}) then
			NextPhase()
		end
		end
	end))
	
	table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_T, function()
	if not IsPaused() then
		if  IsKeyBinding(KEY_T, {KEY_ALT}) then
			MapHide()
		end
		end
	end)) 

  	table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_R, function()
		if not IsPaused() then
		-- Run speed
		if IsKeyBinding(KEY_R, {KEY_CTRL}) then
			ToggleSpeed(self.speed_multiplier)
		-- Map reveal
		elseif IsKeyBinding(KEY_R, {KEY_ALT}) then
			MapReveal()
		end
		end
	end))


	-- Rain
	table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_I, function()
		if not IsPaused() then
		if IsKeyBinding(KEY_I, {KEY_CTRL}) then
			ToggleRain()
		end
		end
	end))
  
  self.inst:StartUpdatingComponent(self)
end)

-------------------------------chromiumboy
function TestToolController:OnLeftClick(down)
    if (TheInput:IsKeyDown(KEY_CTRL) and TheInput:IsKeyDown(KEY_SHIFT)) and down then
        local pt = TheInput:GetWorldPosition()
            
        if IsValidTile(pt) then
            self.inst.Physics:Teleport(pt.x, pt.y, pt.z)
        end
    end
end

function TestToolController:OnRightClick(down)
    if TheInput:IsKeyDown(KEY_SHIFT) and down then
        local pt = self.bookmarked_point
        
        if IsValidTile(pt) then
            self.inst.Physics:Teleport(pt.x, pt.y, pt.z)
        end    
        
    elseif TheInput:IsKeyDown(KEY_CTRL) and down then
        self.bookmarked_point = Vector3(self.inst.Transform:GetWorldPosition())
    end
end



function TestToolController:OnControl(control, down)
    if not IsPaused() then

        if control == CONTROL_PRIMARY then
            self:OnLeftClick(down)
            return
        elseif control == CONTROL_SECONDARY then
            self:OnRightClick(down)
            return
        end
    end
end

-------------------------------chromiumboy
-- Remove handlers if player changes character
function TestToolController:OnRemoveEntity()
    for k,v in pairs(self.inputhandlers) do
        v:Remove()
    end
end

return TestToolController